"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const api_js_1 = require("../api.js");
const queryParams_js_1 = require("../model/queryParams.js");
const helpers_js_1 = require("./helpers.js");
describe("Querying", () => __awaiter(void 0, void 0, void 0, function* () {
    let allPersons;
    let clientWithInactiveEntities;
    before(() => __awaiter(void 0, void 0, void 0, function* () {
        allPersons = (yield helpers_js_1.client.persons.find()).data;
        const defaultConfig = (0, helpers_js_1.getDefaultConfig)();
        defaultConfig.queryOptions = {
            includeInactive: [queryParams_js_1.ActiveEntity.Affiliations, queryParams_js_1.ActiveEntity.Consents],
            includePersonsWithoutChurchAffiliation: true,
        };
        clientWithInactiveEntities = new api_js_1.CoreApiClient(defaultConfig);
    }));
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        yield helpers_js_1.client.refreshTestData();
    }));
    it("serializeFindQueryParams", () => __awaiter(void 0, void 0, void 0, function* () {
        const params = (0, queryParams_js_1.serializeFindQueryParams)({
            sort: [
                {
                    direction: "ascending",
                    path: "firstName",
                },
                "-lastName",
            ],
        });
        chai_1.assert.equal(params.get("sort"), "firstName,-lastName");
    }));
    it("sorting", () => __awaiter(void 0, void 0, void 0, function* () {
        const result = yield helpers_js_1.client.persons.find({
            sort: [
                {
                    direction: "ascending",
                    path: "displayName",
                },
            ],
            limit: 2,
            offset: 1,
        });
        chai_1.assert.equal(result.meta.total, allPersons.length);
        chai_1.assert.equal(result.meta.skipped, 1);
        chai_1.assert.equal(result.meta.limit, 2);
        chai_1.assert.instanceOf(result, api_js_1.WrappedWithMetaArrayPerson);
        const sortedPersons = allPersons.sort((a, b) => a.displayName > b.displayName ? 1 : -1);
        chai_1.assert.equal(result.data[0].uid, sortedPersons[1].uid);
        chai_1.assert.equal(result.data[1].uid, sortedPersons[2].uid);
    }));
    it("filtering", () => __awaiter(void 0, void 0, void 0, function* () {
        const result = yield helpers_js_1.client.persons.find({
            filter: {
                displayName: {
                    _gte: "Philly Daly",
                },
            },
        });
        const expectedResult = allPersons.filter((p) => p.displayName >= "Philly Daly");
        chai_1.assert.equal(result.meta.total, expectedResult.length);
    }));
    it("filtering nested", () => __awaiter(void 0, void 0, void 0, function* () {
        const result = yield helpers_js_1.client.persons.find({
            filter: {
                affiliations: {
                    person: {
                        displayName: {
                            _eq: "Philly Daly",
                        },
                    },
                },
            },
        });
        const expectedResult = allPersons.filter((p) => p.displayName >= "Philly Daly");
        chai_1.assert.equal(result.meta.total, expectedResult.length);
    }));
    it("search - find persons using a fuzzy search", () => __awaiter(void 0, void 0, void 0, function* () {
        const result = yield helpers_js_1.client.persons.find({
            search: "Philli dali",
        });
        chai_1.assert.isDefined(result);
        chai_1.assert.lengthOf(result.data, 1);
        chai_1.assert.equal(result.data[0].uid, helpers_js_1.person1Uid);
    }));
    it("resolving", () => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        const result = yield helpers_js_1.client.persons.find({
            fields: ["*", "affiliations.*"],
        });
        chai_1.assert.instanceOf((_a = result.data[0].affiliations) === null || _a === void 0 ? void 0 : _a[0], api_js_1.Affiliation);
    }));
    it("options - include persons without church affiliation", () => __awaiter(void 0, void 0, void 0, function* () {
        const result = yield helpers_js_1.client.persons.find({
            options: {
                includePersonsWithoutChurchAffiliation: true,
            },
        });
        const resultUids = result.data.map((r) => r.uid);
        chai_1.assert.include(resultUids, helpers_js_1.personInactiveUid);
    }));
    it("options - do not include persons without church affiliation", () => __awaiter(void 0, void 0, void 0, function* () {
        const result = yield helpers_js_1.client.persons.find({
            options: {
                includePersonsWithoutChurchAffiliation: false,
            },
        });
        const resultUids = result.data.map((r) => r.uid);
        chai_1.assert.notInclude(resultUids, helpers_js_1.personInactiveUid);
    }));
    it("options - include inactive entities", () => __awaiter(void 0, void 0, void 0, function* () {
        const result = yield helpers_js_1.client.affiliations.find({
            options: {
                includeInactive: [queryParams_js_1.ActiveEntity.Affiliations],
            },
        });
        const inactiveAffiliation = result.data.find((a) => a.validFrom > new Date() || (a.validTo && a.validTo < new Date()));
        chai_1.assert.isDefined(inactiveAffiliation);
    }));
    it("options - do not include inactive entities", () => __awaiter(void 0, void 0, void 0, function* () {
        const result = yield helpers_js_1.client.affiliations.find();
        const inactiveAffiliation = result.data.find((a) => a.validFrom > new Date() || (a.validTo && a.validTo < new Date()));
        chai_1.assert.isUndefined(inactiveAffiliation);
    }));
    it("options - include inactive by default with correct client", () => __awaiter(void 0, void 0, void 0, function* () {
        const result = yield clientWithInactiveEntities.affiliations.find();
        const inactiveAffiliation = result.data.find((a) => a.validFrom > new Date() || (a.validTo && a.validTo < new Date()));
        chai_1.assert.isDefined(inactiveAffiliation);
    }));
    it("options - include persons without church affiliation with correct client", () => __awaiter(void 0, void 0, void 0, function* () {
        const result = yield clientWithInactiveEntities.persons.get(helpers_js_1.personInactiveUid);
        chai_1.assert.isDefined(result);
        chai_1.assert.equal(result.data.uid, helpers_js_1.personInactiveUid);
    }));
}));
